package com.yeejoin.amos.boot.module.statistcs.biz.controller;

import com.alibaba.fastjson.JSONArray;
import com.yeejoin.amos.boot.module.common.api.dto.DPFilterParamDto;
import com.yeejoin.amos.boot.module.statistcs.biz.service.impl.ZLDPStatisticsServiceImpl;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.typroject.tyboot.core.foundation.enumeration.UserType;
import org.typroject.tyboot.core.restful.doc.TycloudOperation;
import org.typroject.tyboot.core.restful.exception.instance.BadRequest;
import org.typroject.tyboot.core.restful.utils.ResponseHelper;
import org.typroject.tyboot.core.restful.utils.ResponseModel;

import java.util.List;
import java.util.Map;


/**
 * 大屏控制层
 *
 * @author maxy
 * @date 2024-07-09
 */
@RestController
@Api(tags = "大屏-总览统计API")
@RequestMapping(value = "/dp/zl")
public class ZLDPStatisticsController {

    private ZLDPStatisticsServiceImpl statisticsService;

    public ZLDPStatisticsController(ZLDPStatisticsServiceImpl statisticsService) {
        this.statisticsService = statisticsService;
    }


    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @PostMapping(value = "/companyInfo")
    @ApiOperation(httpMethod = "POST", value = "大屏-总览-左屏-企业统计", notes = "大屏-总览-左屏-企业统计")
    public ResponseModel<List<Map<String, Object>>> companyInfo(@RequestBody DPFilterParamDto screenDto) {
        return ResponseHelper.buildResponse(statisticsService.companyInfo(screenDto));
    }

    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @PostMapping(value = "/testOrg")
    @ApiOperation(httpMethod = "POST", value = "大屏-总览-左屏-检测机构统计", notes = "大屏-总览-左屏-检测机构统计")
    public ResponseModel<List<Map<String, Object>>> testOrg(@RequestBody DPFilterParamDto screenDto) {
        return ResponseHelper.buildResponse(statisticsService.testOrg(screenDto));
    }

    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @PostMapping(value = "/userCount")
    @ApiOperation(httpMethod = "POST", value = "大屏-总览-左屏-人员统计", notes = "大屏-总览-左屏-人员统计")
    public ResponseModel<Map<String, Object>> userCount(@RequestBody DPFilterParamDto screenDto) {
        return ResponseHelper.buildResponse(statisticsService.userCount(screenDto));
    }

    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @PostMapping(value = "/userCountNew")
    @ApiOperation(httpMethod = "POST", value = "大屏-总览-左屏-人员统计新", notes = "大屏-总览-左屏-人员统计")
    public ResponseModel<Map<String, Object>> userCountNew(@RequestBody DPFilterParamDto screenDto) {
        return ResponseHelper.buildResponse(statisticsService.userCountNew(screenDto));
    }

    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @PostMapping(value = "/supervisorCount")
    @ApiOperation(httpMethod = "POST", value = "大屏-总览-左屏-行业主管部门统计", notes = "大屏-总览-左屏-行业主管部门统计")
    public ResponseModel<Map<String, Object>> supervisorCount(@RequestBody DPFilterParamDto screenDto) {
        return ResponseHelper.buildResponse(statisticsService.supervisorCount(screenDto));
    }

    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @PostMapping(value = "/equipmentInformCount")
    @ApiOperation(httpMethod = "POST", value = "大屏-总览-左屏-设备使用场所统计", notes = "大屏-总览-左屏-设备使用场所统计")
    public ResponseModel<List<Map<String, Object>>> equipmentInformCount(@RequestBody DPFilterParamDto screenDto) {
        return ResponseHelper.buildResponse(statisticsService.equipmentInformCount(screenDto));
    }

    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @PostMapping (value = "/region/info")
    @ApiOperation(httpMethod = "POST", value = "大屏-总览-安全监管 (近7天)", notes = "安全监管 (近7天)")
    public ResponseModel<JSONArray> statisticsRegionInfo(@Validated @RequestBody DPFilterParamDto dpFilterParamDto, BindingResult result) {
        List<FieldError> fieldErrors = result.getFieldErrors();
        if (!fieldErrors.isEmpty()) {
            throw new BadRequest(fieldErrors.get(0).getDefaultMessage());
        }
        return ResponseHelper.buildResponse(statisticsService.statisticInfoByRegionAndDate(dpFilterParamDto));
    }
}
